
# array of stoplisted words for the DBIx::TextIndex module
# all words must be in lower case and WITHOUT DIACRITICS

# vsechna slova musi byt malymi pismeny a bez diakritiky
# Autor: Tomas Styblo, tripie@cpan.org, 06/2001

package DBIx::TextIndex::stop;

@words = qw(
a
aniz
ano
at
az
bude
budu
by
bych
bychom
byl
byla
bylo
bysme
co
do
i
ja
je
jeho
jeji
jejich
jejim
jemu
ji
jim
jsem
jsi
jsou
k
kam
komu
ma
maji
mam
mela
mel
meli
mit
me
mi
moci
mohl
moje
mu
musel
muset
nac
nam
nas
nase
nasich
nasim
nasemu
ne
nem
nich
o
od
on
ona
ono
presto
pro
proc
proto
protoze
s
se
snad
te
tebe
tem
ti
tebe
ty
u
v
vam
vas
vase
vasich
vasim
vasemu
vedel
vedeli
vedet
vi
vim
vis
za
ze
z
);

1;
